/*
Copyright 1997-2007, Ventek, Inc.
All Rights Reserved
*/
//---------------------------------------------------------------------------

#ifndef VT_TimersH
#define VT_TimersH
//---------------------------------------------------------------------------
#include <QtCore>
//---------------------------------------------------------------------------

/*
CAUTION: access to these timers must be controlled by critical sections
if they are used by more than one thread due to internal state variables.
*/

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
#ifdef Q_WS_WIN

// define minimum Windows support for NT

#ifndef _WIN32_WINNT
#define _WIN32_WINNT 0x0400
#endif

#ifndef WINVER
#define WINVER _WIN32_WINNT
#endif

#include <windows.h>
//---------------------------------------------------------------------------
// high resolution timer object, Win32 Version
class THighResTimer
{
private:
	LARGE_INTEGER t0,t1,f;
	double et,last_dt;
	bool notActive;
protected:
public:
	THighResTimer()
	{
		QueryPerformanceFrequency(&f);
		notActive=f.QuadPart==0;
		HighResReset();
	}

	void HighResReset()   // reset timer
	{
		if (!notActive) QueryPerformanceCounter(&t0);
		last_dt=0.0;
		et=0.0;
	}

	double HighResTime()  // elapsed time in seconds
	{
		double dt;

		if (notActive) return 0.0;

		QueryPerformanceCounter(&t1);
		dt=t1.QuadPart-t0.QuadPart;
		if (dt<0.0)
		{
			et=et+last_dt/f.QuadPart;
			t0=t1;
			last_dt=0.0;
			return et;
		}
		else
		{
			last_dt=dt;
			return et+dt/f.QuadPart;
		}
	}
};

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
// low resolution timer object, Win32 Version
class TLowResTimer
{
private:
	int Time0,TimeE;
protected:
public:
	inline TLowResTimer() { ResetTimer(); }

	void  ResetTimer()		// reset timer
	{
		TimeE=0;
		Time0=GetTickCount();
	}

	double  TimeSeconds()	// elapsed time in seconds
	{
		const int TNow=GetTickCount();
		int E=TNow-Time0;
		if (E<0) E=0;
		TimeE+=E;
		Time0=TNow;
		return TimeE/1000.0;
	}

	int  TimeMSec()				// elapsed time in milli-seconds
	{
		const int TNow=GetTickCount();
		int E=TNow-Time0;
		if (E<0) E=0;
		TimeE+=E;
		Time0=TNow;
		return TimeE;
	}
};
//---------------------------------------------------------------------------

#endif

#ifdef Q_WS_X11

//---------------------------------------------------------------------------
// high resolution timer object, Qt X11 Version
class THighResTimer
{
private:
	QTime t;
protected:
public:
	THighResTimer()
	{
		HighResReset();
	}

	void HighResReset()   // reset timer
	{
		t.start();
	}

	double HighResTime()  // elapsed time in seconds
	{
		return t.elapsed()/1000.0;
	}
};

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
// low resolution timer object, Qt X11 Version
class TLowResTimer
{
private:
	QTime t;
protected:
public:
	inline TLowResTimer() { ResetTimer(); }

	void  ResetTimer()		// reset timer
	{
		t.start();
	}

	double  TimeSeconds()	// elapsed time in seconds
	{
		return t.elapsed()/1000.0;
	}

	int  TimeMSec()				// elapsed time in milli-seconds
	{
		return t.elapsed();
	}
};
//---------------------------------------------------------------------------

#endif

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
// NowUTC returns system time (UTC) to avoid daylight <-> standard time problems
static QDateTime NowUTC(void)
{
#ifdef Q_WS_WIN
	return QDateTime::currentDateTime().toUTC();
	// the disabled code was to see if .toUTC was the cause of the problem
	// it wasn't, but would be slightly faster because .toUTC adds overhead
 // SYSTEMTIME ST;
 // GetSystemTime(&ST);
	//QDate date(ST.wYear,ST.wMonth,ST.wDay);
	//QTime time(ST.wHour,ST.wMinute,ST.wSecond,ST.wMilliseconds);
	//QDateTime dt(date,time,Qt::UTC);
	//return dt;
#endif
#ifdef Q_WS_X11
	return QDateTime::currentDateTime().toUTC();
#endif
}

// convert UTC to local time
static QDateTime UTC_to_Local(const QDateTime UTC)
{
	return UTC.toLocalTime();
}

// convert local time to UTC
static QDateTime Local_to_UTC(const QDateTime Local)
{
	return Local.toUTC();
}
//---------------------------------------------------------------------------
#endif
