//---------------------------------------------------------------------------
#include "qvtk_test_crtl.h"
const int C_PERIOD=5;

Qvtk_Test_CRTL::Qvtk_Test_CRTL(QWidget *parent, Qt::WFlags flags)
: QMainWindow(parent, flags)
{
	ui.setupUi(this);

	runDisplayTimer=new QTimer();

	timeBeginPeriod(C_PERIOD);

	for (int i=0;i<NTHREAD;i++)
	{
		maxTime[i]=0.0;
		crashThreads[i]=new TCrashThread();
		if (i>0)
		{
			crashThreads[i]->pullThread=crashThreads[i-1];
		}
	}

	for (int i=0;i<NTHREAD;i++)
	{
		switch (i)
		{
		case 0:
		case 1:
		case 2:
		case 3:
		case 4:
			crashThreads[i]->StartThread(QThread::TimeCriticalPriority);
			break;
		case 5:
		case 6:
		case 7:
			crashThreads[i]->StartThread(QThread::HighestPriority);
			break;
		case 8:
		case 9:
			crashThreads[i]->StartThread(QThread::HighPriority);
			break;
		case 10:
			crashThreads[i]->StartThread(QThread::NormalPriority);
			break;
		}
	}

	ui.textEdit->clear();

	connect(runDisplayTimer,SIGNAL(timeout()),this,SLOT(runDisplay()));

	runDisplayTimer->start(100);
}

void Qvtk_Test_CRTL::runDisplay()
{
	QString s;
	QTextStream ts(&s);
	ts.setFieldWidth(10);
	ts.setRealNumberPrecision(4);
	ts.setRealNumberNotation(QTextStream::FixedNotation);

	ui.textEdit->clear();

	for (int i=0;i<NTHREAD;i++)
	{
		unsigned int n=crashThreads[i]->GetRunCount();
		double t=crashThreads[i]->GetRunSeconds();
		if (t>maxTime[i]) maxTime[i]=t;
		s.clear();
		ts << i << n << t << maxTime[i] << endl;
		ui.textEdit->insertPlainText(s);
	}
}

Qvtk_Test_CRTL::~Qvtk_Test_CRTL()
{
	runDisplayTimer->stop();

	// stop all threads before deleting them or they will 
	// generate access errors

	for (int i=0;i<NTHREAD;i++)
	{
		crashThreads[i]->StopThread();
	}

	for (int i=0;i<NTHREAD;i++)
	{
		delete crashThreads[i];
	}

	timeEndPeriod(C_PERIOD);
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
TCrashThread::TCrashThread(void):QThread(),
RunCount(0),RunSeconds(0.0),abortRun(false),pullThread(NULL),mutex(NULL)
{
}
//---------------------------------------------------------------------------
TCrashThread::~TCrashThread(void)
{
	StopThread(); // make sure we stop when app exits
	if (mutex!=NULL) delete mutex;	// only delete if assigned
}
//---------------------------------------------------------------------------
void TCrashThread::StartThread(QThread::Priority priority)
{
	// make sure thread is not already running
	if (isRunning()) StopThread();

	// reset control variables
	abortRun=false;
	RunCount=0;
	RunSeconds=0.0;

	// start the thread
	start(priority);
}
//---------------------------------------------------------------------------
void TCrashThread::StopThread(void)
{
	// only stop if running
	if (isRunning())
	{
		abortRun=true;    // set flag

		// wait 10 seconds for thread to stop
		if (!wait(10000))
		{
			// terminate it if it failed to stop
			terminate();
			wait(1000);
		}
	}
}
//---------------------------------------------------------------------------
void TCrashThread::StartRunEvent(void)
{
	// reset control variables
  RunCount++;
	HRT.HighResReset();
}
//---------------------------------------------------------------------------
void TCrashThread::StopRunEvent(void)
{
	// use high res timer
	RunSeconds=HRT.HighResTime();
}
//---------------------------------------------------------------------------
void TCrashThread::run(void)
{
	const int DT=C_PERIOD;
	HANDLE hTE=(HANDLE)CreateWaitableTimer(NULL,false,NULL);

	if (hTE!=NULL)
	{
		LARGE_INTEGER tLT;
		tLT.QuadPart=0;
		if (!SetWaitableTimer(hTE,&tLT,DT,NULL,NULL,true))
		{
			CloseHandle(hTE);
			hTE=NULL;
		}
	}

	// enable this line to see how a mutex will correct the problem

	//mutex=new QMutex();

	while (!abortRun)
	{
		try
		{
			StartRunEvent();

			// access the variables in a loop
			// increase the count from 10 to 20 to increase failure frequency
			// a count of 1 may not fail for a long time

			for (int p=0;p<5;p++)
			{
				// write new values to the objects

				if (mutex!=NULL)
				{
					// use mutex if available
					QMutexLocker mlock(mutex);

					s1=QString::number(rand());
					dt1=NowUTC();
				}
				else
				{
					s1=QString::number(rand());
					dt1=NowUTC();
				}

				// read values from another thread if pointer was set
				// copy them into local objects for force reference use

				if (pullThread!=NULL)
				{
					if (pullThread->mutex!=NULL)
					{
						// use mutex if available
						// must use same mutex of thread where objects exist
						QMutexLocker mlock(pullThread->mutex);

						s2=pullThread->s1;
						dt2=pullThread->dt1;
					}
					else
					{
						// these assignments will cause the crash
						s2=pullThread->s1;
						dt2=pullThread->dt1;
					}
				}
			}

			StopRunEvent();
		}
		catch (...)
		{
		}

		if (hTE==NULL) Sleep(DT);
		else WaitForSingleObject(hTE,DT*4);
	}

	if (hTE!=NULL) CloseHandle(hTE);
}
