#ifndef QVTK_TEST_CRTL_H
#define QVTK_TEST_CRTL_H

#include <QtCore>
#include <QtGui/QMainWindow>
#include "ui_qvtk_test_crtl.h"
//---------------------------------------------------------------------------
// define minimum Windows support for NT

#ifndef _WIN32_WINNT
#define _WIN32_WINNT 0x0400
#endif

#ifndef WINVER
#define WINVER _WIN32_WINNT
#endif

#include <windows.h>
#include <mmsystem.h>
#include <stdlib.h>
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
// high resolution timer object, Win32 Version
class THighResTimer
{
private:
	LARGE_INTEGER t0,t1,f;
	double et,last_dt;
	bool notActive;
protected:
public:
	THighResTimer()
	{
		QueryPerformanceFrequency(&f);
		notActive=f.QuadPart==0;
		HighResReset();
	}

	void HighResReset()   // reset timer
	{
		if (!notActive) QueryPerformanceCounter(&t0);
		last_dt=0.0;
		et=0.0;
	}

	double HighResTime()  // elapsed time in seconds
	{
		double dt;

		if (notActive) return 0.0;

		QueryPerformanceCounter(&t1);
		dt=t1.QuadPart-t0.QuadPart;
		if (dt<0.0)
		{
			et=et+last_dt/f.QuadPart;
			t0=t1;
			last_dt=0.0;
			return et;
		}
		else
		{
			last_dt=dt;
			return et+dt/f.QuadPart;
		}
	}
};

//---------------------------------------------------------------------------
// NowUTC returns system time (UTC) to avoid daylight <-> standard time problems
static QDateTime NowUTC(void)
{
	return QDateTime::currentDateTime().toUTC();
}


class TCrashThread:public QThread
{
	Q_OBJECT
private:
  // these allow monitoring of operation
  unsigned int RunCount;
	THighResTimer HRT;
	double RunSeconds;
protected:
	virtual void run(void);
	bool abortRun;			// stops thread
public:
	TCrashThread *pullThread;	// thread to copy objects from

	// this mutex is used to test protection of the objects

	QMutex *mutex;

	// these objects are implicitly shared

	QDateTime dt1;	// our time
	QDateTime dt2;	// copied time from pullThread

	QString s1;	// our string
	QString s2;	// copied string from pullThread

	TCrashThread(void);
	~TCrashThread(void);

	// call to start the thread
  virtual void StartThread(QThread::Priority priority
		=QThread::InheritPriority);

	// call to stop the thread
  virtual void StopThread(void);

	// call to log start run event
  virtual void StartRunEvent(void);

	// call to log stop run event
  virtual void StopRunEvent(void);

	// methods for thread monitoring
  unsigned int GetRunCount(void) { return RunCount; };
	double GetRunSeconds(void) { return RunSeconds; };
};

const int NTHREAD=11;	// number of threads to run

class Qvtk_Test_CRTL : public QMainWindow
{
	Q_OBJECT

public:
	Qvtk_Test_CRTL(QWidget *parent = 0, Qt::WFlags flags = 0);
	~Qvtk_Test_CRTL();

private:
	Ui::Qvtk_Test_CRTLClass ui;

	TCrashThread *crashThreads[NTHREAD];

	double maxTime[NTHREAD];

	QTimer *runDisplayTimer;

	private slots:
		void runDisplay();
};

#endif // QVTK_TEST_CRTL_H
